@@ -13,7 +13,7 @@ use 5.006;
 use Exporter;
 
 BEGIN {
-    $VERSION = '1.10_03';
+    $VERSION = '1.10_04';
     @EXPORT  = qw( Dump Load DumpFile LoadFile );
     @ISA     = qw( Exporter );
 
@@ -169,6 +169,13 @@ B<NOTE>: If you are working with other language's YAML/Syck bindings
 calling the C<Load>/C<Dump> functions.  The default setting is for
 preserving backward-compatibility with C<YAML.pm>.
 
+=head1 Differences Between Yaml::Syck and YAML
+
+head2 Error handling
+
+Some calls are designed to die rather than returning YAML. You should wrap
+your calls in eval to assure you do not get unexpected results.
+
 =head1 FLAGS
 
 =head2 $YAML::Syck::Headless
@@ -226,7 +233,9 @@ blessing tag names that do not begin with C<!!perl> or C<!perl>; see below.
 
 =head1 BUGS
 
-Dumping Glob/IO values does not work yet.
+Dumping Glob/IO values do not work yet.
+
+Dumping of Tied variables is unsupported.
 
 =head1 CAVEATS
 
